@@ -54,8 +54,8 @@ downloads, installation and source code.
 
 __Conventions__  = 'CF-1.6'
 __author__       = 'David Hassell'
-__date__         = '2017-07-18'
-__version__      = '2.0.1.post2'
+__date__         = '2017-07-24'
+__version__      = '2.0.2'
 
 from distutils.version import StrictVersion
 import imp
@@ -79,7 +79,6 @@ else:
 from .variable             import Variable
 from .boundedvariable      import BoundedVariable, Bounds
 from .coordinate           import Coordinate, DimensionCoordinate, AuxiliaryCoordinate
-#from .coordinatebounds     import CoordinateBounds
 from .coordinatereference  import CoordinateReference
 from .cellmeasure          import CellMeasure
 from .domainancillary      import DomainAncillary
@@ -88,7 +87,6 @@ from .field                import Field, FieldList
 from .fieldancillary       import FieldAncillary
 from .read                 import read, read1
 from .write                import write
-from .utils                import Dict
 from .units                import Units
 from .cfdatetime           import Datetime, dt
 from .timeduration         import TimeDuration, Y, M, D, h, m, s
