@@ -1242,7 +1242,8 @@ Cell methods    : time: minimum within years
 
                 axis = cell_methods.pop(0)[:-1]
                 if field is not None:
-                    axis = field.axis(axis)
+                    axis = field.axis(axis, key=True)
+
                 axes.append(axis)
             #--- End: while
             cm.axes  = axes
@@ -1976,7 +1977,7 @@ Translate netCDF variable names stored in the `!names` attribute into
                 else:                    
                     # name must (ought to) be a standard name
                     axes.append(field.axis({'standard_name': name},
-                                           role='d', exact=True))
+                                           role='d', exact=True, key=True))
             #--- End: for
 
             cm.names = tuple(names)
@@ -2074,7 +2075,7 @@ Translate `names` to CF-netCDF names.
 #                cm.axes = (None,)
 #                continue
 #                
-#            cm.axes = tuple([(f.axis(name)
+#            cm.axes = tuple([(f.axis(name, key=True)
 #                              if axis is None or override else
 #                              axis)
 #                             for axis, name in zip(cm.axes, names)])
