@@ -19,7 +19,7 @@ def read(files, verbose=False, ignore_read_error=False,
          fmt=None, select=None, select_options={}, field=None,
          height_at_top_of_model=None, recursive=False,
          follow_symlinks=False, um=None, chunk=True, _debug=False):
-    '''Read fields from files into `cf.Field` objects.
+    '''Read fields from netCDF, PP or UM fields files.
 
 A file may be on disk or on a OPeNDAP server.
 
@@ -528,7 +528,7 @@ Read the contents of a single file into a field list.
                          verbose=verbose, set_standard_name=True,
                          height_at_top_of_model=height_at_top_of_model,
                          fmt=fmt, word_size=word_size, endian=endian,
-                         chunk=chunk)
+                         chunk=chunk, _debug=_debug)
 
         # PP fields are aggregated intrafile prior to interfile
         # aggregation
