@@ -7,11 +7,11 @@ def write(fields, filename, fmt='NETCDF3_CLASSIC', overwrite=True,
           single=False, double=False, reference_datetime=None,
           variable_attributes=None, HDF_chunksizes=None,
           unlimited=None, _debug=False):
-    '''Write fields to a CF-netCDF or CFA-netCDF file.
+    '''Write fields to a netCDF file.
     
-NetCDF dimension and variable names will be taken from variables'
-`~Variable.ncvar` attributes and the domain attribute
-`~Domain.nc_dimensions` if present, otherwise they are inferred from
+NetCDF dimension and variable names will be taken, if present, from
+variables' `~Variable.ncvar` attributes and the domain axis
+`~DomainAxis.ncdim` attributes, otherwise they are inferred from
 standard names or set to defaults. NetCDF names may be automatically
 given a numerical suffix to avoid duplication.
 
@@ -19,10 +19,6 @@ Output netCDF file global properties are those which occur in the set
 of CF global properties and non-standard data variable properties and
 which have equal values across all input fields.
 
-Logically identical field components are only written to the file
-once, apart from when they need to fulfil both dimension coordinate
-and auxiliary coordinate roles for different data variables.
-
 .. seealso:: `cf.read`
 
 :Parameters:
