@@ -32,13 +32,13 @@ class DataTest(unittest.TestCase):
     
     test_only = []
 #    test_only = ['NOTHING!!!!!']
-#    test_only = ['test_Data_AUXILIARY_MASK']
 #    test_only = ['test_Data__asdatetime__asreftime__isdatetime']
+#    test_only = ['test_Data_AUXILIARY_MASK']
 #    test_only = ['test_Data__collapse_UNWEIGHTED_MASKED']
 #    test_only = ['test_Data__collapse_UNWEIGHTED_UNMASKED']
 #    test_only = ['test_Data__collapse_WEIGHTED_UNMASKED']
 #    test_only = ['test_Data__collapse_WEIGHTED_MASKED']
-#    test_only = ['test_Data_any']
+#    test_only = ['test_Data_flip']
 #    test_only = ['test_Data_mean']
 #    test_only = ['test_Data_sample_size']
 #    test_only = ['test_Data_section']
@@ -80,6 +80,111 @@ class DataTest(unittest.TestCase):
                          for n in range(1, a.ndim+1)
                          for axes in itertools.permutations(range(a.ndim), n)]
 
+    def test_Data_AUXILIARY_MASK(self):
+        if self.test_only and inspect.stack()[0][3] not in self.test_only:
+            return
+
+        d = cf.Data()
+        self.assertTrue(d._auxiliary_mask is None)
+        self.assertTrue(d._auxiliary_mask_return() is None)
+
+        d = cf.Data.empty((90, 60))
+        m = numpy.full(d.shape, False, bool)
+
+        self.assertTrue(d._auxiliary_mask is None)
+        self.assertTrue(d._auxiliary_mask_return().shape == m.shape)
+        self.assertTrue((d._auxiliary_mask_return() == m).all())
+        self.assertTrue(d._auxiliary_mask is None)
+
+        m[[0, 2, 80], [0, 40, 20]] = True
+
+        d._auxiliary_mask_add_component(cf.Data(m))
+        self.assertTrue(len(d._auxiliary_mask) == 1)
+        self.assertTrue(d._auxiliary_mask_return().shape == m.shape)
+        self.assertTrue((d._auxiliary_mask_return() == m).all())
+
+        d = cf.Data.empty((90, 60))
+        m = numpy.full(d.shape, False, bool)
+
+        d = cf.Data.empty((90, 60))
+        d._auxiliary_mask_add_component(cf.Data(m[0:1, :]))
+        self.assertTrue(len(d._auxiliary_mask) == 1)
+        self.assertTrue((d._auxiliary_mask_return() == m).all())
+
+        d = cf.Data.empty((90, 60))
+        d._auxiliary_mask_add_component(cf.Data(m[:, 0:1]))
+        self.assertTrue(len(d._auxiliary_mask) == 1)
+        self.assertTrue((d._auxiliary_mask_return() == m).all())
+
+        d = cf.Data.empty((90, 60))
+        d._auxiliary_mask_add_component(cf.Data(m[:, 0:1]))
+        d._auxiliary_mask_add_component(cf.Data(m[0:1, :]))
+        self.assertTrue(len(d._auxiliary_mask) == 2)
+        self.assertTrue(d._auxiliary_mask_return().shape == m.shape)
+        self.assertTrue((d._auxiliary_mask_return() == m).all())
+
+        for chunksize in self.chunk_sizes:   
+            cf.CHUNKSIZE(chunksize) 
+
+            # --------------------------------------------------------
+            d = cf.Data(numpy.arange(120).reshape(30, 4))
+            e = cf.Data(numpy.arange(120, 280).reshape(40, 4))
+            
+            fm = cf.Data.full((70, 4), False, bool)
+            fm[ 0, 0] = True
+            fm[10, 2] = True
+            fm[20, 1] = True
+
+            dm = fm[:30]
+            d._auxiliary_mask = [dm]
+            
+            f = cf.Data.concatenate([d, e], axis=0)
+            self.assertTrue(f.shape == fm.shape)
+            
+            self.assertTrue((f._auxiliary_mask_return().array == fm).all())
+
+            # --------------------------------------------------------
+            d = cf.Data(numpy.arange(120).reshape(30, 4))
+            e = cf.Data(numpy.arange(120, 280).reshape(40, 4))
+
+            fm = cf.Data.full((70, 4), False, bool)
+            fm[50, 0] = True
+            fm[60, 2] = True
+            fm[65, 1] = True
+
+            em = fm[30:]
+            e._auxiliary_mask = [em]
+
+            f = cf.Data.concatenate([d, e], axis=0)
+            self.assertTrue(f.shape == fm.shape)
+            
+            self.assertTrue((f._auxiliary_mask_return().array == fm).all())
+
+            # --------------------------------------------------------
+            d = cf.Data(numpy.arange(120).reshape(30, 4))
+            e = cf.Data(numpy.arange(120, 280).reshape(40, 4))
+
+            fm = cf.Data.full((70, 4), False, bool)
+            fm[ 0, 0] = True
+            fm[10, 2] = True
+            fm[20, 1] = True
+            fm[50, 0] = True
+            fm[60, 2] = True
+            fm[65, 1] = True
+
+            dm = fm[:30]
+            d._auxiliary_mask = [dm]
+            em = fm[30:]
+            e._auxiliary_mask = [em]
+
+            f = cf.Data.concatenate([d, e], axis=0)
+            self.assertTrue(f.shape == fm.shape)
+            
+            self.assertTrue((f._auxiliary_mask_return().array == fm).all())
+
+        cf.CHUNKSIZE(self.original_chunksize)
+    #--- End: for
+    
     def test_Data___contains__(self):
         if self.test_only and inspect.stack()[0][3] not in self.test_only:
             return
@@ -101,6 +206,10 @@ class DataTest(unittest.TestCase):
         cf.CHUNKSIZE(self.original_chunksize)
     #--- End: def
 
+    def test_Data___getitem__(self):
+        if self.test_only and inspect.stack()[0][3] not in self.test_only:
+            return
+        
     def test_Data___setitem__(self):
         if self.test_only and inspect.stack()[0][3] not in self.test_only:
             return
@@ -465,6 +574,32 @@ class DataTest(unittest.TestCase):
 #            print 'mshape =', d._pmshape
         #--- End: def
         cf.CHUNKSIZE(self.original_chunksize)
+
+        array = numpy.arange(3*4*5).reshape(3, 4, 5) + 1
+        d = cf.Data(array.copy(), 'metre', chunk=False)
+        d.chunk(total=[0], omit_axes=[1, 2])
+
+        self.assertTrue(d._pmshape == (3,))
+        self.assertTrue(d[0].shape  == (1, 4, 5))
+        self.assertTrue(d[-1].shape == (1, 4, 5))
+        self.assertTrue(d[0].max()  == 4*5)
+        self.assertTrue(d[-1].max() == 3*4*5)
+
+        for i in (2, 1):
+            e = d.flip(i)
+            self.assertTrue(e._pmshape == (3,))
+            self.assertTrue(e[0].shape  == (1, 4, 5))
+            self.assertTrue(e[-1].shape == (1, 4, 5))
+            self.assertTrue(e[0].max()  == 4*5)
+            self.assertTrue(e[-1].max() == 3*4*5)
+        
+        i = 0
+        e = d.flip(i)
+        self.assertTrue(e._pmshape == (3,))
+        self.assertTrue(e[0].shape  == (1, 4, 5))
+        self.assertTrue(e[-1].shape == (1, 4, 5))
+        self.assertTrue(e[0].max()  == 3*4*5)
+        self.assertTrue(e[-1].max() == 4*5)        
     #--- End: def
 
     def test_Data_max(self):
