@@ -13,13 +13,16 @@ class FieldTest(unittest.TestCase):
                                      'test_file.nc')
         self.filename2 = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                                       'test_file2.nc')
+        self.contiguous = os.path.join(os.path.dirname(os.path.abspath(__file__)),
+                                       'DSG_timeSeries_contiguous.nc')
+        
         self.chunk_sizes = (17, 34, 300, 100000)[::-1]
         self.original_chunksize = cf.CHUNKSIZE()
         self.f = cf.read1(self.filename)
 
         self.test_only = []
+#        self.test_only = ['test_Field_AUXILIARY_MASK']
 #        self.test_only = ['test_Field___add__']
-#        self.test_only = ['test_Field_collapse']
 #        self.test_only = ['test_Field_transpose','test_Field_squeeze']
 #        self.test_only = ['test_Field_match','test_Field_items']
 #        self.test_only = ['test_Field_items']
@@ -28,8 +31,175 @@ class FieldTest(unittest.TestCase):
 #        self.test_only = ['test_Field_anchor']
 #        self.test_only = ['test_Field_period']
 #        self.test_only = ['test_Field_mask_invalid']
-#        self.test_only = ['test_Field___setitem__']
+#        self.test_only = ['test_Field___getitem__']
 #        self.test_only = ['test_Field_section']
+#        self.test_only = ['test_Field_expand_dims']
+
+    def test_Field_AUXILIARY_MASK(self):
+        if self.test_only and inspect.stack()[0][3] not in self.test_only:
+            return
+
+        ac = numpy.ma.masked_all((3, 7))
+        ac[0, 0:5] = [1.0, 2.0, 3.0, -99, 5.0]
+        ac[0, 3  ] = numpy.ma.masked
+        ac[1, 1:5] =      [1.5, 2.5, 3.5, 4.5]
+        ac[2, 3:7] =                [1.0, 2.0, 3.0, 5.0]
+        
+        ae = numpy.ma.masked_all((3, 8))
+        ae[0, 0:5] = [1.0, 2.0, 3.0, -99, 5.0]
+        ae[0, 3  ] = numpy.ma.masked
+        ae[1, 1:5] =      [1.5, 2.5, 3.5, 4.5]
+        ae[2, 3:8] =                [1.0, 2.0, 3.0, -99, 5.0]
+        ae[2, 6  ] = numpy.ma.masked
+
+        af = numpy.ma.masked_all((4, 9))
+        af[1, 0:5] = [1.0, 2.0, 3.0, -99, 5.0]
+        af[1, 3  ] = numpy.ma.masked
+        af[2, 1:5] =      [1.5, 2.5, 3.5, 4.5]
+        af[3, 3:8] =                [1.0, 2.0, 3.0, -99, 5.0]
+        af[3, 6  ] = numpy.ma.masked
+        
+        query1 = cf.wi(1, 5) & cf.ne(4)
+
+        for chunksize in self.chunk_sizes:
+            cf.CHUNKSIZE(chunksize)
+            f = cf.read(self.contiguous)[0]
+            
+            for (method, shape, a) in zip(['compress', 'envelope', 'full'],
+                                          [ac.shape, ae.shape, af.shape],
+                                          [ac, ae, af]):
+                message = 'method={!r}'.format(method)
+
+                g = f.subspace(method, time=query1)
+                t = g.coord('time')
+
+                self.assertTrue(g.shape == shape, message)
+                self.assertTrue(t.shape == shape, message)
+
+                self.assertTrue((t.data._auxiliary_mask_return().array == a.mask).all(), message)
+                self.assertTrue((g.data._auxiliary_mask_return().array == a.mask).all(), message)
+
+                self.assertTrue(cf.functions._numpy_allclose(t.array, a), message)
+        #--- End: for
+        cf.CHUNKSIZE(self.original_chunksize)
+
+        query2 = cf.set([1, 3, 5])
+
+        ac2 = numpy.ma.masked_all((2, 6))
+        ac2[0, 0] = 1
+        ac2[0, 1] = 3
+        ac2[0, 3] = 5
+        ac2[1, 2] = 1
+        ac2[1, 4] = 3
+        ac2[1, 5] = 5
+
+        ae2 = numpy.ma.where((ae==1)| (ae==3) | (ae==5), ae, numpy.ma.masked)
+        af2 = numpy.ma.where((af==1)| (af==3) | (af==5), af, numpy.ma.masked)
+
+        for chunksize in self.chunk_sizes:
+            cf.CHUNKSIZE(chunksize)
+            f = cf.read(self.contiguous)[0]
+            
+            for (method, shape, a) in zip(['compress', 'envelope', 'full'],
+                                          [ac2.shape, ae2.shape, af2.shape],
+                                          [ac2, ae2, af2]):
+
+                message = 'method={!r}'.format(method)
+
+                h = f.subspace('full', time=query1)
+                g = h.subspace(method, time=query2)
+                t = g.coord('time')
+    
+                self.assertTrue(g.shape == shape, message)        
+                self.assertTrue(t.shape == shape, message)
+                            
+                self.assertTrue((t.data._auxiliary_mask_return().array == a.mask).all(), message)
+                self.assertTrue((g.data._auxiliary_mask_return().array == a.mask).all(), message)
+                
+                self.assertTrue(cf.functions._numpy_allclose(t.array, a), message)
+        #--- End: for
+        cf.CHUNKSIZE(self.original_chunksize)
+
+        ac3 = numpy.ma.masked_all((2, 3))
+        ac3[0, 0] = -2
+        ac3[1, 1] = 3
+        ac3[1, 2] = 4
+          
+        ae3 = numpy.ma.masked_all((3, 6))
+        ae3[0, 0]  = -2
+        ae3[2, 4] = 3
+        ae3[2, 5] = 4
+          
+        af3 = numpy.ma.masked_all((3, 8))
+        af3[0, 0] = -2
+        af3[2, 4] = 3
+        af3[2, 5] = 4
+        
+        query3 = cf.set([-2, 3, 4])
+
+        for chunksize in self.chunk_sizes:
+            cf.CHUNKSIZE(chunksize)
+            f = cf.read(self.contiguous)[0].subspace[[0, 2, 3], 1:]
+            
+            for (method, shape, a) in zip(['compress', 'envelope', 'full'],
+                                          [ac3.shape, ae3.shape, af3.shape],
+                                          [ac3, ae3, af3]):
+
+                message = 'method={!r}'.format(method)
+
+                g = f.subspace(method, time=query3)
+                t = g.coord('time')
+    
+                self.assertTrue(g.shape == shape, message)        
+                self.assertTrue(t.shape == shape, message)
+                            
+                self.assertTrue((t.data._auxiliary_mask_return().array == a.mask).all(), message)
+                self.assertTrue((g.data._auxiliary_mask_return().array == a.mask).all(), message)
+                
+                self.assertTrue(cf.functions._numpy_allclose(t.array, a), message)
+        #--- End: for
+        cf.CHUNKSIZE(self.original_chunksize)
+#--- End: def
+
+    def test_Field___getitem__(self):
+        if self.test_only and inspect.stack()[0][3] not in self.test_only:
+            return
+
+        for chunksize in self.chunk_sizes:
+            cf.CHUNKSIZE(chunksize)
+            f = cf.read1(self.filename).squeeze()
+            d = f.data
+            f = cf.read1(self.filename).squeeze()
+
+            g = f[...]
+            self.assertTrue((g.data == d).all())
+
+            g = f[:, :]
+            self.assertTrue((g.data == d).all())
+
+            g = f[slice(None), :]
+            self.assertTrue((g.data == d).all())
+
+            g = f[:, slice(0, f.shape[1], 1)]
+            self.assertTrue((g.data == d).all())
+
+            g = f[slice(0, None, 1), slice(0, None)]
+            self.assertTrue((g.data == d).all())
+
+            g = f[3:7, 2:5]
+            self.assertTrue((g.data == d[3:7, 2:5]).all())
+            
+            g = f[6:2:-1, 4:1:-1]
+            self.assertTrue((g.data == d[6:2:-1, 4:1:-1]).all())
+            
+            g = f[[0, 3, 8], [1, 7, 8]]
+
+            g = f[[8, 3, 0], [8, 7, 1]]
+
+            g = f[[7, 4, 1], slice(6, 8)]
+        #--- End: for    
+        cf.CHUNKSIZE(self.original_chunksize)
+    #--- End: def
 
     def test_Field___setitem__(self):
         if self.test_only and inspect.stack()[0][3] not in self.test_only:
@@ -126,16 +296,25 @@ class FieldTest(unittest.TestCase):
         if self.test_only and inspect.stack()[0][3] not in self.test_only:
             return
 
+
         f = self.f
-        self.assertTrue(f.axes() == {'dim0':1, 'dim1':10, 'dim2':9})
+
+        da = {'dim0': 1, 'dim1': 10, 'dim2': 9}
+
+        self.assertTrue(f.axes() == da)
+
+        self.assertTrue(f.axes() == {'dim0': cf.DomainAxis(1), 'dim1': cf.DomainAxis(10), 'dim2': cf.DomainAxis(9)})
+
+        for key in f.data_axes():
+            self.assertTrue(f.axis(key).size == da[key])
 
         for i in range(f.ndim):
-            self.assertTrue(f.axis(i) == f.data_axes()[i])
+            self.assertTrue(f.axis(i, key=True) == f.data_axes()[i])
 
         self.assertTrue(set(f.axes(slice(0,3))) == set(f.data_axes()))
 
         for k, v in f.ncdimensions.iteritems():
-            self.assertTrue(f.axis('ncdim%' + v) == k)
+            self.assertTrue(f.axis('ncdim%'+v, key=True) == k)
     #--- End: def
 
     def test_Field_data_axes(self):
@@ -163,6 +342,30 @@ class FieldTest(unittest.TestCase):
         cf.CHUNKSIZE(self.original_chunksize)
     #--- End: def
 
+    def test_Field_expand_dims(self):
+        if self.test_only and inspect.stack()[0][3] not in self.test_only:
+            return
+
+        f = cf.read1(self.filename)
+        
+        g = f.copy()   
+        g.expand_dims(i=1)
+        self.assertTrue(g.ndim == f.ndim + 1)
+        self.assertTrue(g.data_axes()[1:] == f.data_axes())
+
+        g = f.expand_dims(0)
+        self.assertTrue(g.ndim == f.ndim + 1)
+        self.assertTrue(g.data_axes()[1:] == f.data_axes())
+
+        g = f.expand_dims(3)
+        self.assertTrue(g.ndim == f.ndim + 1)
+        self.assertTrue(g.data_axes()[:-1] == f.data_axes())
+
+        g = f.expand_dims(-3)
+        self.assertTrue(g.ndim == f.ndim + 1)
+        self.assertTrue(g.data_axes()[1:] == f.data_axes())
+    #--- End: def
+
 #    def test_Field_indices(self):
 #        if self.test_only and inspect.stack()[0][3] not in self.test_only:
 #            return
@@ -337,8 +540,8 @@ class FieldTest(unittest.TestCase):
 
         for chunksize in self.chunk_sizes:
             cf.CHUNKSIZE(chunksize)
-            f = cf.read1(self.filename2)
-            self.assertTrue(len(f.section(('X','Y'))) == 1800,
+            f = cf.read1(self.filename2)[0:100]
+            self.assertTrue(len(f.section(('X','Y'))) == 100,
                             'CHUNKSIZE = %s' % chunksize)
         #--- End: for
         cf.CHUNKSIZE(self.original_chunksize)
@@ -383,67 +586,67 @@ class FieldTest(unittest.TestCase):
         cf.CHUNKSIZE(self.original_chunksize)
     #--- End: def
 
-    def test_Field_collapse(self):
-        if self.test_only and inspect.stack()[0][3] not in self.test_only:
-            return
-
-        for chunksize in self.chunk_sizes:
-            cf.CHUNKSIZE(chunksize)
-            f = cf.read(self.filename2)[0]
-
-            g = f.collapse('mean')
-            self.assertTrue(g.cell_methods == cf.CellMethods('time: maximum time: latitude: longitude: mean').write(f.axes_names()))
-
-            g = f.collapse('mean', axes=['T', 'X'])
-            self.assertTrue(g.cell_methods == cf.CellMethods('time: maximum time: longitude: mean').write(f.axes_names()))
-
-            g = f.collapse('mean', axes=[0, 2])
-            self.assertTrue(g.cell_methods == cf.CellMethods('time: maximum time: longitude: mean').write(f.axes_names()))
-            
-            g = f.collapse('T: mean within years time: minimum over years', 
-                           within_years=cf.M(), weights=None, _debug=0)
-            self.assertTrue(g.cell_methods == cf.CellMethods('time: maximum time: mean within years time: minimum over years').write(f.axes_names()))
-
-            for m in range(1, 13):
-                a = numpy.empty((5, 4, 5))
-                for i, year in enumerate(f.subspace(T=cf.month(m)).coord('T').year.unique()):
-                    q = cf.month(m) & cf.year(year)
-                    x = f.subspace(T=q)
-                    x.data.mean(axes=0, i=True)
-                    a[i] = x.array
-                #--- End: for
-                a = a.min(axis=0)
-                self.assertTrue(numpy.allclose(a, g.array[m % 12]))
-            #--- End: for  
-
-            g = f.collapse('T: mean', group=360)
-
-            for group in (cf.M(12), 
-                          cf.M(12, month=12),
-                          cf.M(12, day=16),
-                          cf.M(12, month=11, day=27)):
-                g = f.collapse('T: mean', group=group)
-                bound = g.coord('T').bounds.dtarray[0, 1]
-                self.assertTrue(bound.month == group.offset.month,
-                                "{}!={}, group={}".format(bound.month, group.offset.month, group))
-                self.assertTrue(bound.day   == group.offset.day,
-                                "{}!={}, group={}".format(bound.day, group.offset.day, group))
-            #--- End: for  
-
-#            for group in (cf.D(30), 
-#                          cf.D(30, month=12),
-#                          cf.D(30, day=16),
-#                          cf.D(30, month=11, day=27)):
+#    def test_Field_collapse(self):
+#        if self.test_only and inspect.stack()[0][3] not in self.test_only:
+#            return
+#
+#        for chunksize in self.chunk_sizes:
+#            cf.CHUNKSIZE(chunksize)
+#            f = cf.read(self.filename2)[0]
+#
+#            g = f.collapse('mean')
+#            self.assertTrue(g.cell_methods == cf.CellMethods('time: maximum time: latitude: longitude: mean').write(f.axes_names()))
+#
+#            g = f.collapse('mean', axes=['T', 'X'])
+#            self.assertTrue(g.cell_methods == cf.CellMethods('time: maximum time: longitude: mean').write(f.axes_names()))
+#
+#            g = f.collapse('mean', axes=[0, 2])
+#            self.assertTrue(g.cell_methods == cf.CellMethods('time: maximum time: longitude: mean').write(f.axes_names()))
+#            
+#            g = f.collapse('T: mean within years time: minimum over years', 
+#                           within_years=cf.M(), weights=None, _debug=0)
+#            self.assertTrue(g.cell_methods == cf.CellMethods('time: maximum time: mean within years time: minimum over years').write(f.axes_names()))
+#
+#            for m in range(1, 13):
+#                a = numpy.empty((5, 4, 5))
+#                for i, year in enumerate(f.subspace(T=cf.month(m)).coord('T').year.unique()):
+#                    q = cf.month(m) & cf.year(year)
+#                    x = f.subspace(T=q)
+#                    x.data.mean(axes=0, i=True)
+#                    a[i] = x.array
+#                #--- End: for
+#                a = a.min(axis=0)
+#                self.assertTrue(numpy.allclose(a, g.array[m % 12]))
+#            #--- End: for  
+#
+#            g = f.collapse('T: mean', group=360)
+#
+#            for group in (cf.M(12), 
+#                          cf.M(12, month=12),
+#                          cf.M(12, day=16),
+#                          cf.M(12, month=11, day=27)):
 #                g = f.collapse('T: mean', group=group)
 #                bound = g.coord('T').bounds.dtarray[0, 1]
-#                self.assertTrue(bound.day == group.offset.day,
-#                                "{}!={}, bound={}, group={}".format(bound.day, group.offset.day, bound, group))
-            #--- End: for  
-
-        #--- End: for    
-
-        cf.CHUNKSIZE(self.original_chunksize)
-    #--- End: def
+#                self.assertTrue(bound.month == group.offset.month,
+#                                "{}!={}, group={}".format(bound.month, group.offset.month, group))
+#                self.assertTrue(bound.day   == group.offset.day,
+#                                "{}!={}, group={}".format(bound.day, group.offset.day, group))
+#            #--- End: for  
+#
+##            for group in (cf.D(30), 
+##                          cf.D(30, month=12),
+##                          cf.D(30, day=16),
+##                          cf.D(30, month=11, day=27)):
+##                g = f.collapse('T: mean', group=group)
+##                bound = g.coord('T').bounds.dtarray[0, 1]
+##                self.assertTrue(bound.day == group.offset.day,
+##                                "{}!={}, bound={}, group={}".format(bound.day, group.offset.day, bound, group))
+#            #--- End: for  
+#
+#        #--- End: for    
+#
+#        cf.CHUNKSIZE(self.original_chunksize)
+#    #--- End: def
 
     def test_Field_where(self):
         if self.test_only and inspect.stack()[0][3] not in self.test_only:
