@@ -26,8 +26,14 @@ class ppTest(unittest.TestCase):
     def setUp(self):
         self.ppfilename = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                                        'wgdos_packed.pp')
+
         self.new_table = os.path.join(os.path.dirname(os.path.abspath(__file__)),
-                                      'new_STASH_to_CF.txt')        
+                                      'new_STASH_to_CF.txt')
+
+        text_file = open(self.new_table, 'w')
+        text_file.write('1!24!SURFACE TEMPERATURE AFTER TIMESTEP  !Pa!!!NEW_NAME!!')
+        text_file.close()
+        
         self.chunk_sizes = (17, 34, 300, 100000)[::-1]
         self.original_chunksize = cf.CHUNKSIZE()
         self.test_only = ()
@@ -51,9 +57,6 @@ class ppTest(unittest.TestCase):
     #--- End: def
 
     def test_PP_WGDOS_UNPACKING(self):
-#        print 'skipping WGDOS unpacking !!!!!!! ... '
-#        return
-        cf.CHUNKSIZE(10000000)
         f = cf.read1(self.ppfilename)
 
         self.assertTrue(f.min() > 221.71,
@@ -78,7 +81,7 @@ class ppTest(unittest.TestCase):
                 self.assertTrue(f.equals(g, traceback=True),
                                 'Bad writing/reading. format='+fmt)
         #--- End: for
-        original_chunksize = cf.CHUNKSIZE()
+        cf.CHUNKSIZE(self.original_chunksize)
     #--- End: def
 
 #--- End: class
