@@ -1,11 +1,14 @@
+import datetime
 import tempfile
-import cf
 import os
 import unittest
 import atexit
-import numpy
 import inspect
 
+import numpy
+
+import cf
+
 tmpfile  = tempfile.mktemp('.cf-python_test')
 tmpfiles = [tmpfile]
 def _remove_tmpfiles():
@@ -29,8 +32,10 @@ class read_writeTest(unittest.TestCase):
 #    test_only = ['NOTHING!!!!!']
 #    test_only = ['test_write_reference_datetime']
 #    test_only = ['test_write_HDF_chunks']
-    test_only = ['test_read_write_unlimited']
-
+#    test_only = ['test_read_write_unlimited']
+#    test_only = ['test_read_field']
+#    test_only = ['test_write_datatype']
+    
     def test_read_select(self):
         if self.test_only and inspect.stack()[0][3] not in self.test_only:
             return
@@ -42,26 +47,24 @@ class read_writeTest(unittest.TestCase):
                         'Bad read with select keyword')
     #--- End: def
 
-    def test_read_top_level(self):
+    def test_read_field(self):
         if self.test_only and inspect.stack()[0][3] not in self.test_only:
             return
 
-        # Test top_level keyword of cf.read
+        # Test field keyword of cf.read
         filename = self.filename
         self.assertTrue(len(cf.read(filename)) == 1)
-        self.assertTrue(len(cf.read(filename, top_level=['dimension'])) == 6)
-        self.assertTrue(len(cf.read(filename, top_level=['auxiliary'])) == 11)
-        self.assertTrue(len(cf.read(filename, top_level='measure')) == 4)
-        self.assertTrue(len(cf.read(filename, top_level=['ancillary'])) == 5)
-        self.assertTrue(len(cf.read(filename, top_level='reference')) == 2)
-        self.assertTrue(len(cf.read(filename, top_level='field')) == 6)
-        self.assertTrue(len(cf.read(filename, top_level=['ancillary', 'auxiliary'])) == 15)
-        self.assertTrue(len(cf.read(filename, top_level=['reference', 'auxiliary'])) == 12)
-        self.assertTrue(len(cf.read(filename, top_level=['field', 'auxiliary'])) == 16)
-        self.assertTrue(len(cf.read(filename, top_level=['field', 'measure', 'auxiliary'])) == 19)
-        self.assertTrue(len(cf.read(filename, top_level='coordinate')) == 16)
-        self.assertTrue(len(cf.read(filename, top_level='all')) == 24)
-        self.assertTrue(len(cf.read(filename, top_level=('field', 'measure', 'coordinate'))) == 24)
+#        print cf.read(filename)
+        self.assertTrue(len(cf.read(filename, field=['dimension'])) == 4)
+        self.assertTrue(len(cf.read(filename, field=['auxiliary'])) == 4)
+        self.assertTrue(len(cf.read(filename, field='measure')) == 2)
+        self.assertTrue(len(cf.read(filename, field=['field_ancillary'])) == 5)
+        self.assertTrue(len(cf.read(filename, field='domain_ancillary')) == 4)
+        self.assertTrue(len(cf.read(filename, field=['field_ancillary', 'auxiliary'])) == 8)
+        self.assertTrue(len(cf.read(filename, field=['domain_ancillary', 'auxiliary'])) == 7)
+        self.assertTrue(len(cf.read(filename, field=['domain_ancillary', 'measure', 'auxiliary'])) == 8)
+        self.assertTrue(len(cf.read(filename, field='all')) == 15)
+        self.assertTrue(len(cf.read(filename, field=('field_ancillary', 'dimension', 'measure', 'auxiliary', 'domain_ancillary'))) == 15)
     #--- End: def
 
     def test_read_write_format(self):
@@ -117,7 +120,7 @@ class read_writeTest(unittest.TestCase):
             self.assertTrue(f.dtype == numpy.dtype(float))
             cf.write(f, tmpfile, fmt='NETCDF4', 
                      datatype={numpy.dtype(float): numpy.dtype('float32')})
-            g = cf.read(tmpfile)
+            g = cf.read1(tmpfile)
             self.assertTrue(g.dtype == numpy.dtype('float32'), 
                             'datatype read in is '+str(g.dtype))
         #--- End: for
@@ -134,10 +137,10 @@ class read_writeTest(unittest.TestCase):
                 f = cf.read(self.filename)[0]
                 t = cf.DimensionCoordinate(data=cf.Data(123, 'days since 1750-1-1'))
                 t.standard_name = 'time'
-                dim = f.insert_axis(1)
+                dim = f.insert_axis(cf.DomainAxis(1))
                 f.insert_dim(t, key=dim)
                 cf.write(f, tmpfile, fmt='NETCDF4', reference_datetime=reference_datetime)
-                g = cf.read(tmpfile)
+                g = cf.read1(tmpfile)
                 t = g.dim('T')
                 self.assertTrue(t.Units == cf.Units('days since '+reference_datetime),
                                 'Units written were '+repr(t.Units.reftime)+' not '+repr(reference_datetime))
@@ -170,8 +173,8 @@ class read_writeTest(unittest.TestCase):
             cf.write(f, tmpfile, fmt=fmt)
             f.unlimited(org)
             
-            g = cf.read(tmpfile)
-            self.assertTrue(g.unlimited()[g.axis(axis)] is True,
+            g = cf.read1(tmpfile)
+            self.assertTrue(g.unlimited()[g.axis(axis, key=True)] is True,
                             'Failed with axis={}, fmt={}'.format(axis, fmt))
 
         fmt = 'NETCDF3_CLASSIC'
@@ -180,8 +183,8 @@ class read_writeTest(unittest.TestCase):
             cf.write(f, tmpfile, fmt=fmt)
             f.unlimited(org)
             
-            g = cf.read(tmpfile)
-            self.assertTrue(g.unlimited()[g.axis(axis)] is True,
+            g = cf.read1(tmpfile)
+            self.assertTrue(g.unlimited()[g.axis(axis, key=True)] is True,
                             'Failed with axis={}, fmt={}'.format(axis, fmt))
 
         fmt = 'NETCDF4'
@@ -189,10 +192,10 @@ class read_writeTest(unittest.TestCase):
         cf.write(f, tmpfile, fmt=fmt)
         f.unlimited(org)
 
-        g = cf.read(tmpfile)
-        self.assertTrue(g.unlimited()[g.axis('X')] is True,
+        g = cf.read1(tmpfile)
+        self.assertTrue(g.unlimited()[g.axis('X', key=True)] is True,
                         'Failed with axis={}, fmt={}'.format('X', fmt))
-        self.assertTrue(g.unlimited()[g.axis('Y')] is True,
+        self.assertTrue(g.unlimited()[g.axis('Y', key=True)] is True,
                         'Failed with axis={}, fmt={}'.format('Y', fmt))
 
 
@@ -201,8 +204,8 @@ class read_writeTest(unittest.TestCase):
         cf.write(f, tmpfile, fmt=fmt, unlimited=['X'])
         f.unlimited(org)
 
-        g = cf.read(tmpfile)
-        self.assertTrue(not g.unlimited()[g.axis('X')],
+        g = cf.read1(tmpfile)
+        self.assertTrue(not g.unlimited()[g.axis('X', key=True)],
                         'Failed with axis={}, fmt={}'.format('X', fmt))
 
 
@@ -211,8 +214,8 @@ class read_writeTest(unittest.TestCase):
         cf.write(f, tmpfile, fmt=fmt, unlimited=['X'])
         f.unlimited(org)
 
-        g = cf.read(tmpfile)
-        self.assertTrue(g.unlimited()[g.axis('X')] is True,
+        g = cf.read1(tmpfile)
+        self.assertTrue(g.unlimited()[g.axis('X', key=True)] is True,
                         'Failed with axis={}, fmt={}'.format('X', fmt))
 
 
@@ -221,10 +224,10 @@ class read_writeTest(unittest.TestCase):
         cf.write(f, tmpfile, fmt=fmt, unlimited=['X'])
         f.unlimited(org)
 
-        g = cf.read(tmpfile)
-        self.assertTrue(g.unlimited()[g.axis('X')] is True,
+        g = cf.read1(tmpfile)
+        self.assertTrue(g.unlimited()[g.axis('X', key=True)] is True,
                         'Failed with axis={}, fmt={}'.format('X', fmt))
-        self.assertTrue(g.unlimited()[g.axis('Y')] is True,
+        self.assertTrue(g.unlimited()[g.axis('Y', key=True)] is True,
                         'Failed with axis={}, fmt={}'.format('Y', fmt))
 
  
@@ -234,10 +237,10 @@ class read_writeTest(unittest.TestCase):
         cf.write(f, tmpfile, fmt=fmt)
         f.unlimited(org)
 
-        g = cf.read(tmpfile)
-        self.assertTrue(g.unlimited()[g.axis('X')] is True,
+        g = cf.read1(tmpfile)
+        self.assertTrue(g.unlimited()[g.axis('X', key=True)] is True,
                         'Failed with axis={}, fmt={}'.format('X', fmt))
-        self.assertTrue(g.unlimited()[g.axis('Y')] is True,
+        self.assertTrue(g.unlimited()[g.axis('Y', key=True)] is True,
                         'Failed with axis={}, fmt={}'.format('Y', fmt))
 
  
@@ -248,17 +251,17 @@ class read_writeTest(unittest.TestCase):
         cf.write(f, tmpfile, fmt=fmt)
         f.unlimited(org)
 
-        g = cf.read(tmpfile)
-        self.assertTrue(not g.unlimited()[g.axis('X')],
+        g = cf.read1(tmpfile)
+        self.assertTrue(not g.unlimited()[g.axis('X', key=True)],
                         'Failed with axis={}, fmt={}'.format('X', fmt))
-        self.assertTrue(not g.unlimited()[g.axis('Y')],
+        self.assertTrue(not g.unlimited()[g.axis('Y', key=True)],
                         'Failed with axis={}, fmt={}'.format('Y', fmt))
 
     #--- End: def
 #--- End: class
 
 if __name__ == "__main__":
-    print 'cf-python version:', cf.__version__
-    print 'cf-python path:'   , os.path.abspath(cf.__file__)
-    print ''
+    print 'Run date:', datetime.datetime.now()
+    cf.environment()
+    print''
     unittest.main(verbosity=2)
